const express = require('express');

const libs = `${process.cwd()}/libs`;

const squel = require('squel');

const log = require(`${libs}/log`)(module);

const query = require(`${libs}/middlewares/query`);

const sqlQuery = require(`${libs}/db/query_exec`);

const response = require(`${libs}/middlewares/response`);

const locationApp = express();

function locationIdToStr(locationId) {
    let locationStr = 'Total';
    switch(locationId) {
        case 1:
            locationStr = 'Urbana';                         break;
        case 2:
            locationStr = 'Rural';                          break;
        case 3:
            locationStr = 'Área de assentamento';           break;
        case 4:
            locationStr = 'Terra indígena';                 break;
        case 5:
            locationStr = 'Área remanescente de quilombos'; break;
        case 6:
            locationStr = 'Unidade de uso sustentável';     break;

    }
    return locationStr;
}

locationApp.get('/sociodemographic', (req, res, next) => {
    const populationYearQry = squel.select()
        .field('MAX(ibge_populacao.ano_censo)')
        .from('ibge_populacao');

    const populationQry = squel.select()
        .field('\'Brasil\'', 'name')
        .field('SUM(populacao)', 'population')
        .field('ibge_populacao.ano_censo')
        .from('ibge_populacao')
        .where(`ibge_populacao.ano_censo IN (${populationYearQry.toString()})`)
        .group('ibge_populacao.ano_censo');

    const pibYearQry = squel.select()
        .field('MAX(ibge_pib.ano_censo)')
        .from('ibge_pib');

    const pibQry = squel.select()
        .field('\'Brasil\'', 'name')
        .field('AVG(ibge_pib.pib_per_capita)', 'gdp_per_capita')
        .field('ibge_pib.ano_censo')
        .from('ibge_pib')
        .where(`ibge_pib.ano_censo IN (${pibYearQry.toString()})`)
        .group('ibge_pib.ano_censo');

    const idhYearQry = squel.select()
        .field('MAX(adh_idh.ano_censo)')
        .from('adh_idh');

    const idhQry = squel.select()
        .field('\'Brasil\'', 'name')
        .field('AVG(idhm)', 'idhm')
        .field('adh_idh.ano_censo')
        .from('adh_idh')
        .where(`adh_idh.ano_censo IN (${idhYearQry.toString()})`)
        .group('adh_idh.ano_censo');

    const analfabYearQry = squel.select()
        .field('MAX(adh_analfabetismo.ano_censo)')
        .from('adh_analfabetismo');

    const analfabQry = squel.select()
        .field('\'Brasil\'', 'name')
        .field('AVG(t_analf15m)', 'analfabetism')
        .field('adh_analfabetismo.ano_censo')
        .from('adh_analfabetismo')
        .where(`adh_analfabetismo.ano_censo IN (${analfabYearQry.toString()})`)
        .group('adh_analfabetismo.ano_censo');

    const giniYearQry = squel.select()
        .field('MAX(adh_gini.ano_censo)')
        .from('adh_gini');

    const giniQry = squel.select()
        .field('\'Brasil\'', 'name')
        .field('AVG(gini)', 'gini')
        .field('adh_gini.ano_censo')
        .from('adh_gini')
        .where(`adh_gini.ano_censo IN (${giniYearQry.toString()})`)
        .group('adh_gini.ano_censo');

    const queries = [populationQry, pibQry, idhQry, analfabQry, giniQry].map((qry) => {
        return sqlQuery(qry.toString());
    });

    // execute all queries concurrently
    Promise.all(queries).then((queryResults) => {
        req.result = [];
        for(let result of queryResults) {
            log.debug(result);
            req.result.push(result[0]);
        }
        next();
    }).catch((error) => {
        log.error(`[SQL query error] ${error}`);
        next(error);
    });
}, response('location'));

locationApp.get('/sociodemographic/region/:id', (req, res, next) => {
    const regionId = parseInt(req.params.id, 10);

    const populationYearQry = squel.select()
        .field('MAX(ibge_populacao.ano_censo)')
        .from('ibge_populacao');

    const populationQry = squel.select()
        .field('regiao.nome', 'name')
        .field('SUM(populacao)', 'population')
        .field('ano_censo', 'census_year')
        .from('ibge_populacao')
        .from('municipio')
        .from('estado')
        .from('regiao')
        .where(`regiao.pk_regiao_id = ${regionId}`)
        .where(`ibge_populacao.ano_censo IN (${populationYearQry.toString()})`)
        .where('ibge_populacao.fk_municipio_id = municipio.pk_cod_ibge')
        .where('municipio.fk_estado_id = estado.pk_estado_id')
        .where('estado.fk_regiao_id = regiao.pk_regiao_id')
        .group('regiao.nome')
        .group('ibge_populacao.ano_censo')
        .order('regiao.nome');

    const pibYearQry = squel.select()
        .field('MAX(ibge_pib.ano_censo)')
        .from('ibge_pib');

    const pibQry = squel.select()
        .field('regiao.nome', 'name')
        .field('AVG(ibge_pib.pib_per_capita)', 'gdp_per_capita')
        .field('ano_censo', 'census_year')
        .from('ibge_pib')
        .from('municipio')
        .from('estado')
        .from('regiao')
        .where(`regiao.pk_regiao_id = ${regionId}`)
        .where(`ibge_pib.ano_censo IN (${pibYearQry.toString()})`)
        .where('ibge_pib.fk_municipio_id = municipio.pk_cod_ibge')
        .where('municipio.fk_estado_id = estado.pk_estado_id')
        .where('estado.fk_regiao_id = regiao.pk_regiao_id')
        .group('regiao.nome')
        .group('ibge_pib.ano_censo')
        .order('regiao.nome');

    const idhYearQry = squel.select()
        .field('MAX(adh_idh.ano_censo)')
        .from('adh_idh');

    const idhQry = squel.select()
        .field('regiao.nome', 'name')
        .field('AVG(idhm)', 'idhm')
        .field('ano_censo', 'census_year')
        .from('adh_idh')
        .from('municipio')
        .from('estado')
        .from('regiao')
        .where(`regiao.pk_regiao_id = ${regionId}`)
        .where(`adh_idh.ano_censo IN (${idhYearQry.toString()})`)
        .where('adh_idh.fk_municipio_id = municipio.pk_cod_ibge')
        .where('municipio.fk_estado_id = estado.pk_estado_id')
        .where('estado.fk_regiao_id = regiao.pk_regiao_id')
        .group('regiao.nome')
        .group('adh_idh.ano_censo')
        .order('regiao.nome');

    const analfabYearQry = squel.select()
        .field('MAX(adh_analfabetismo.ano_censo)')
        .from('adh_analfabetismo');

    const analfabQry = squel.select()
        .field('regiao.nome', 'name')
        .field('AVG(t_analf15m)', 'analfabetism')
        .field('ano_censo', 'census_year')
        .from('adh_analfabetismo')
        .from('municipio')
        .from('estado')
        .from('regiao')
        .where(`regiao.pk_regiao_id = ${regionId}`)
        .where(`adh_analfabetismo.ano_censo IN (${analfabYearQry.toString()})`)
        .where('adh_analfabetismo.fk_municipio_id = municipio.pk_cod_ibge')
        .where('municipio.fk_estado_id = estado.pk_estado_id')
        .where('estado.fk_regiao_id = regiao.pk_regiao_id')
        .group('regiao.nome')
        .group('adh_analfabetismo.ano_censo')
        .order('regiao.nome');

    const giniYearQry = squel.select()
        .field('MAX(adh_gini.ano_censo)')
        .from('adh_gini');

    const giniQry = squel.select()
        .field('regiao.nome', 'name')
        .field('AVG(gini)', 'gini')
        .field('ano_censo', 'census_year')
        .from('adh_gini')
        .from('municipio')
        .from('estado')
        .from('regiao')
        .where(`regiao.pk_regiao_id = ${regionId}`)
        .where(`adh_gini.ano_censo IN (${giniYearQry.toString()})`)
        .where('adh_gini.fk_municipio_id = municipio.pk_cod_ibge')
        .where('municipio.fk_estado_id = estado.pk_estado_id')
        .where('estado.fk_regiao_id = regiao.pk_regiao_id')
        .group('regiao.nome')
        .group('adh_gini.ano_censo')
        .order('regiao.nome');

    const queries = [populationQry, pibQry, idhQry, analfabQry, giniQry].map((qry) => {
        return sqlQuery(qry.toString());
    });

    // execute all queries concurrently
    Promise.all(queries).then((queryResults) => {
        req.result = [];
        for(let result of queryResults) {
            log.debug(result);
            req.result.push(result[0]);
        }
        next();
    }).catch((error) => {
        log.error(`[SQL query error] ${error}`);
        next(error);
    });
}, response('location'));

locationApp.get('/sociodemographic/state/:id', (req, res, next) => {
    const stateId = parseInt(req.params.id, 10);

    const populationYearQry = squel.select()
        .field('MAX(ibge_populacao.ano_censo)')
        .from('ibge_populacao');
    // load all cities by state and compute the sociodemographic and educational information
    const populationQry = squel.select()
        .field('estado.nome', 'name')
        .field('SUM(populacao)', 'population')
        .field('ibge_populacao.ano_censo', 'census_year')
        .from('ibge_populacao')
        .from('municipio')
        .from('estado')
        .where(`estado.pk_estado_id = ${stateId}`)
        .where(`ibge_populacao.ano_censo IN (${populationYearQry.toString()})`)
        .where('ibge_populacao.fk_municipio_id = municipio.pk_cod_ibge')
        .where('municipio.fk_estado_id = estado.pk_estado_id')
        .group('estado.nome')
        .group('ibge_populacao.ano_censo')
        .order('estado.nome');

    const pibYearQry = squel.select()
        .field('MAX(ibge_pib.ano_censo)')
        .from('ibge_pib');

    const pibQry = squel.select()
        .field('estado.nome', 'name')
        .field('AVG(ibge_pib.pib_per_capita)', 'gdp_per_capita')
        .field('ibge_pib.ano_censo', 'census_year')
        .from('ibge_pib')
        .from('municipio')
        .from('estado')
        .where(`estado.pk_estado_id = ${stateId}`)
        .where(`ibge_pib.ano_censo IN (${pibYearQry.toString()})`)
        .where('ibge_pib.fk_municipio_id = municipio.pk_cod_ibge')
        .where('municipio.fk_estado_id = estado.pk_estado_id')
        .group('estado.nome')
        .group('ibge_pib.ano_censo')
        .order('estado.nome');

    const idhYearQry = squel.select()
        .field('MAX(adh_idh.ano_censo)')
        .from('adh_idh');

    const idhQry = squel.select()
        .field('estado.nome', 'name')
        .field('AVG(idhm)', 'idhm')
        .field('adh_idh.ano_censo', 'census_year')
        .from('adh_idh')
        .from('municipio')
        .from('estado')
        .where(`estado.pk_estado_id = ${stateId}`)
        .where(`adh_idh.ano_censo IN (${idhYearQry.toString()})`)
        .where('adh_idh.fk_municipio_id = municipio.pk_cod_ibge')
        .where('municipio.fk_estado_id = estado.pk_estado_id')
        .group('estado.nome')
        .group('adh_idh.ano_censo')
        .order('estado.nome');

    const analfabYearQry = squel.select()
        .field('MAX(adh_analfabetismo.ano_censo)')
        .from('adh_analfabetismo');

    const analfabQry = squel.select()
        .field('estado.nome', 'name')
        .field('AVG(t_analf15m)', 'analfabetism')
        .field('adh_analfabetismo.ano_censo', 'census_year')
        .from('adh_analfabetismo')
        .from('municipio')
        .from('estado')
        .where(`estado.pk_estado_id = ${stateId}`)
        .where(`adh_analfabetismo.ano_censo IN (${analfabYearQry.toString()})`)
        .where('adh_analfabetismo.fk_municipio_id = municipio.pk_cod_ibge')
        .where('municipio.fk_estado_id = estado.pk_estado_id')
        .group('estado.nome')
        .group('adh_analfabetismo.ano_censo')
        .order('estado.nome');

    const giniYearQry = squel.select()
        .field('MAX(adh_gini.ano_censo)')
        .from('adh_gini');

    const giniQry = squel.select()
        .field('estado.nome', 'name')
        .field('AVG(gini)', 'gini')
        .field('adh_gini.ano_censo', 'census_year')
        .from('adh_gini')
        .from('municipio')
        .from('estado')
        .where(`estado.pk_estado_id = ${stateId}`)
        .where(`adh_gini.ano_censo IN (${giniYearQry.toString()})`)
        .where('adh_gini.fk_municipio_id = municipio.pk_cod_ibge')
        .where('municipio.fk_estado_id = estado.pk_estado_id')
        .group('estado.nome')
        .group('adh_gini.ano_censo')
        .order('estado.nome');

    const queries = [populationQry, pibQry, idhQry, analfabQry, giniQry].map((qry) => {
        return sqlQuery(qry.toString());
    });

    // execute all queries concurrently
    Promise.all(queries).then((queryResults) => {
        req.result = [];
        for(let result of queryResults) {
            log.debug(result);
            req.result.push(result[0]);
        }
        next();
    }).catch((error) => {
        log.error(`[SQL query error] ${error}`);
        next(error);
    });
}, response('location'));

locationApp.get('/sociodemographic/city/:id', (req, res, next) => {
    const cityId = parseInt(req.params.id, 10);

    const populationYearQry = squel.select()
        .field('MAX(ibge_populacao.ano_censo)')
        .from('ibge_populacao');
    // load all cities by state and compute the sociodemographic and educational information
    const populationQry = squel.select()
        .field('municipio.nome', 'name')
        .field('SUM(populacao)', 'population')
        .field('ibge_populacao.ano_censo', 'census_year')
        .from('ibge_populacao')
        .from('municipio')
        .where(`municipio.pk_cod_ibge = ${cityId}`)
        .where(`ibge_populacao.ano_censo IN (${populationYearQry.toString()})`)
        .where('ibge_populacao.fk_municipio_id = municipio.pk_cod_ibge')
        .group('municipio.nome')
        .group('ibge_populacao.ano_censo')
        .order('municipio.nome');

    const pibYearQry = squel.select()
        .field('MAX(ibge_pib.ano_censo)')
        .from('ibge_pib');

    const pibQry = squel.select()
        .field('municipio.nome', 'name')
        .field('AVG(ibge_pib.pib_per_capita)', 'gdp_per_capita')
        .field('ibge_pib.ano_censo', 'census_year')
        .from('ibge_pib')
        .from('municipio')
        .where(`municipio.pk_cod_ibge = ${cityId}`)
        .where(`ibge_pib.ano_censo IN (${pibYearQry.toString()})`)
        .where('ibge_pib.fk_municipio_id = municipio.pk_cod_ibge')
        .group('municipio.nome')
        .group('ibge_pib.ano_censo')
        .order('municipio.nome');

    const idhYearQry = squel.select()
        .field('MAX(adh_idh.ano_censo)')
        .from('adh_idh');

    const idhQry = squel.select()
        .field('municipio.nome', 'name')
        .field('AVG(idhm)', 'idhm')
        .field('adh_idh.ano_censo', 'census_year')
        .from('adh_idh')
        .from('municipio')
        .where(`municipio.pk_cod_ibge = ${cityId}`)
        .where(`adh_idh.ano_censo IN (${idhYearQry.toString()})`)
        .where('adh_idh.fk_municipio_id = municipio.pk_cod_ibge')
        .group('municipio.nome')
        .group('adh_idh.ano_censo')
        .order('municipio.nome');

    const analfabYearQry = squel.select()
        .field('MAX(adh_analfabetismo.ano_censo)')
        .from('adh_analfabetismo');

    const analfabQry = squel.select()
        .field('municipio.nome', 'name')
        .field('AVG(t_analf15m)', 'analfabetism')
        .field('adh_analfabetismo.ano_censo', 'census_year')
        .from('adh_analfabetismo')
        .from('municipio')
        .where(`municipio.pk_cod_ibge = ${cityId}`)
        .where(`adh_analfabetismo.ano_censo IN (${analfabYearQry.toString()})`)
        .where('adh_analfabetismo.fk_municipio_id = municipio.pk_cod_ibge')
        .group('municipio.nome')
        .group('adh_analfabetismo.ano_censo')
        .order('municipio.nome');

    const giniYearQry = squel.select()
        .field('MAX(adh_gini.ano_censo)')
        .from('adh_gini');

    const giniQry = squel.select()
        .field('municipio.nome', 'name')
        .field('AVG(gini)', 'gini')
        .field('adh_gini.ano_censo', 'census_year')
        .from('adh_gini')
        .from('municipio')
        .where(`municipio.pk_cod_ibge = ${cityId}`)
        .where(`adh_gini.ano_censo IN (${giniYearQry.toString()})`)
        .where('adh_gini.fk_municipio_id = municipio.pk_cod_ibge')
        .group('municipio.nome')
        .group('adh_gini.ano_censo')
        .order('municipio.nome');

    const queries = [populationQry, pibQry, idhQry, analfabQry, giniQry].map((qry) => {
        return sqlQuery(qry.toString());
    });

    // execute all queries concurrently
    Promise.all(queries).then((queryResults) => {
        req.result = [];
        for(let result of queryResults) {
            log.debug(result);
            req.result.push(result[0]);
        }
        next();
    }).catch((error) => {
        log.error(`[SQL query error] ${error}`);
        next(error);
    });
}, response('location'));

locationApp.get('/educational', (req, res, next) => {
    const censusYearQry = squel.select()
        .field('MAX(escola.ano_censo)', 'ano_censo')
        .from('escola')
        .toString();

    const totalSchoolsQry = squel.select()
        .field('\'Brasil\'', 'name')
        .field('0', 'location')
        .field('COUNT(DISTINCT(escola.cod_entidade))', 'total')
        .field('escola.ano_censo', 'census_year')
        .from('escola')
        .where(`escola.ano_censo IN (${censusYearQry})`)
        .where('escola.id_tipo_turma = 0')
        .group('escola.ano_censo');

    const schoolsPerLocationQry = squel.select()
        .field('\'Brasil\'', 'name')
        .field('escola.tipo_localizacao', 'location')
        .field('COUNT(DISTINCT(escola.cod_entidade))', 'total')
        .field('escola.ano_censo', 'census_year')
        .from('escola')
        .where(`escola.ano_censo IN (${censusYearQry})`)
        .where('escola.id_tipo_turma = 0')
        .group('escola.tipo_localizacao')
        .group('escola.ano_censo');

    const schoolClassYearQry = squel.select()
        .field('MAX(turma.ano_censo)')
        .from('turma')
        .toString();

    const enrollmentsQry = squel.select()
        .field('\'Brasil\'', 'name')
        .field('COALESCE(SUM(turma.num_matriculas), 0)', 'total')
        .field('turma.ano_censo', 'census_year')
        .field('\'Total\'', 'adm_dependency')
        .from('turma')
        .where(`turma.ano_censo IN (${schoolClassYearQry})`)
        .where('turma.fk_tipo_turma_id <= 3')
        .group('turma.ano_censo');

    const enrollmentsPerAdmDepQry = squel.select()
        .field('\'Brasil\'', 'name')
        .field('COALESCE(SUM(turma.num_matriculas), 0)', 'total')
        .field('turma.ano_censo', 'census_year')
        .field('dependencia_adm.nome', 'adm_dependency')
        .from('turma')
        .from('dependencia_adm')
        .where('turma.fk_dependencia_adm_id = dependencia_adm.pk_dependencia_adm_id')
        .where('turma.fk_tipo_turma_id <= 3')
        .where(`turma.ano_censo IN (${schoolClassYearQry})`)
        .group('turma.ano_censo')
        .group('dependencia_adm.nome');

    const enrollmentsPerSchoolLevelQry = squel.select()
        .field('\'Brasil\'', 'name')
        .field('COALESCE(SUM(turma.num_matriculas), 0)', 'total')
        .field('turma.ano_censo', 'census_year')
        .field('dependencia_adm.nome', 'adm_dependency')
        .field('etapa_ensino.desc_etapa', 'school_level')
        .from('turma')
        .from('dependencia_adm')
        .from('etapa_ensino')
        .where('turma.fk_dependencia_adm_id = dependencia_adm.pk_dependencia_adm_id')
        .where('turma.fk_etapa_ensino_id = etapa_ensino.pk_etapa_ensino_id')
        .where('turma.fk_tipo_turma_id <= 3')
        .where(`turma.ano_censo IN (${schoolClassYearQry})`)
        .group('turma.ano_censo')
        .group('dependencia_adm.nome')
        .group('etapa_ensino.desc_etapa')
        .order('dependencia_adm.nome')
        .order('etapa_ensino.desc_etapa');

    const queries = [totalSchoolsQry, schoolsPerLocationQry, enrollmentsQry, enrollmentsPerAdmDepQry, enrollmentsPerSchoolLevelQry].map((qry) => {
        return sqlQuery(qry.toString());
    });

    // execute all queries concurrently
    Promise.all(queries).then((queryResults) => {
        req.result = [];
        for(let result of queryResults) {
            log.debug(result);
            for(let row of result) {
                row.location = locationIdToStr(row.location);
                req.result.push(row);
            }
        }
        next();
    }).catch((error) => {
        log.error(`[SQL query error] ${error}`);
        next(error);
    });
}, response('location'));

locationApp.get('/educational/region/:id', (req, res, next) => {
    const regionId = parseInt(req.params.id, 10);

    const censusYearQry = squel.select()
        .field('MAX(escola.ano_censo)', 'ano_censo')
        .from('escola')
        .toString();

    const totalSchoolsQry = squel.select()
        .field('regiao.nome', 'name')
        .field('0', 'location')
        .field('COUNT(DISTINCT(escola.cod_entidade))', 'total')
        .field('escola.ano_censo', 'census_year')
        .from('escola')
        .from('estado')
        .from('regiao')
        .where('escola.fk_estado_id = estado.pk_estado_id')
        .where('estado.fk_regiao_id = regiao.pk_regiao_id')
        .where(`regiao.pk_regiao_id = ${regionId}`)
        .where(`escola.ano_censo IN (${censusYearQry})`)
        .where('escola.id_tipo_turma = 0')
        .group('regiao.nome')
        .group('escola.ano_censo');

    const schoolsPerLocationQry = squel.select()
        .field('regiao.nome', 'name')
        .field('escola.tipo_localizacao', 'location')
        .field('COUNT(DISTINCT(escola.cod_entidade))', 'total')
        .field('escola.ano_censo', 'census_year')
        .from('escola')
        .from('estado')
        .from('regiao')
        .where('escola.fk_estado_id = estado.pk_estado_id')
        .where('estado.fk_regiao_id = regiao.pk_regiao_id')
        .where(`regiao.pk_regiao_id = ${regionId}`)
        .where(`escola.ano_censo IN (${censusYearQry})`)
        .where('escola.id_tipo_turma = 0')
        .group('regiao.nome')
        .group('escola.tipo_localizacao')
        .group('escola.ano_censo');

    const schoolClassYearQry = squel.select()
        .field('MAX(turma.ano_censo)')
        .from('turma')
        .toString();

    const enrollmentsQry = squel.select()
        .field('regiao.nome', 'name')
        .field('COALESCE(SUM(turma.num_matriculas), 0)', 'total')
        .field('turma.ano_censo', 'census_year')
        .field('\'Total\'', 'adm_dependency')
        .from('turma')
        .from('regiao')
        .where('turma.fk_regiao_id = regiao.pk_regiao_id')
        .where(`turma.fk_regiao_id = ${regionId}`)
        .where(`turma.ano_censo IN (${schoolClassYearQry})`)
        .where('turma.fk_tipo_turma_id <= 3')
        .group('turma.ano_censo')
        .group('regiao.nome');

    const enrollmentsPerAdmDepQry = squel.select()
        .field('regiao.nome', 'name')
        .field('COALESCE(SUM(turma.num_matriculas), 0)', 'total')
        .field('turma.ano_censo', 'census_year')
        .field('dependencia_adm.nome', 'adm_dependency')
        .from('turma')
        .from('dependencia_adm')
        .from('regiao')
        .where('turma.fk_regiao_id = regiao.pk_regiao_id')
        .where(`turma.fk_regiao_id = ${regionId}`)
        .where('turma.fk_dependencia_adm_id = dependencia_adm.pk_dependencia_adm_id')
        .where('turma.fk_tipo_turma_id <= 3')
        .where(`turma.ano_censo IN (${schoolClassYearQry})`)
        .group('turma.ano_censo')
        .group('dependencia_adm.nome')
        .group('regiao.nome');

    const enrollmentsPerSchoolLevelQry = squel.select()
        .field('regiao.nome', 'name')
        .field('COALESCE(SUM(turma.num_matriculas), 0)', 'total')
        .field('turma.ano_censo', 'census_year')
        .field('dependencia_adm.nome', 'adm_dependency')
        .field('etapa_ensino.desc_etapa', 'school_level')
        .from('turma')
        .from('dependencia_adm')
        .from('etapa_ensino')
        .from('regiao')
        .where('turma.fk_regiao_id = regiao.pk_regiao_id')
        .where(`turma.fk_regiao_id = ${regionId}`)
        .where('turma.fk_dependencia_adm_id = dependencia_adm.pk_dependencia_adm_id')
        .where('turma.fk_etapa_ensino_id = etapa_ensino.pk_etapa_ensino_id')
        .where('turma.fk_tipo_turma_id <= 3')
        .where(`turma.ano_censo IN (${schoolClassYearQry})`)
        .group('turma.ano_censo')
        .group('dependencia_adm.nome')
        .group('etapa_ensino.desc_etapa')
        .group('regiao.nome')
        .order('regiao.nome')
        .order('dependencia_adm.nome')
        .order('etapa_ensino.desc_etapa');

    const queries = [totalSchoolsQry, schoolsPerLocationQry, enrollmentsQry, enrollmentsPerAdmDepQry, enrollmentsPerSchoolLevelQry].map((qry) => {
        return sqlQuery(qry.toString());
    });

    // execute all queries concurrently
    Promise.all(queries).then((queryResults) => {
        req.result = [];
        for(let result of queryResults) {
            log.debug(result);
            for(let row of result) {
                row.location = locationIdToStr(row.location);
                req.result.push(row);
            }
        }
        next();
    }).catch((error) => {
        log.error(`[SQL query error] ${error}`);
        next(error);
    });
}, response('location'));

locationApp.get('/educational/state/:id', (req, res, next) => {
    const stateId = parseInt(req.params.id, 10);

    const censusYearQry = squel.select()
        .field('MAX(escola.ano_censo)', 'ano_censo')
        .from('escola')
        .toString();

    const totalSchoolsQry = squel.select()
        .field('estado.nome', 'name')
        .field('0', 'location')
        .field('COUNT(DISTINCT(escola.cod_entidade))', 'total')
        .field('escola.ano_censo', 'census_year')
        .from('escola')
        .from('estado')
        .where('escola.fk_estado_id = estado.pk_estado_id')
        .where(`escola.fk_estado_id = ${stateId}`)
        .where(`escola.ano_censo IN (${censusYearQry})`)
        .where('escola.id_tipo_turma = 0')
        .group('estado.nome')
        .group('escola.ano_censo');

    const schoolsPerLocationQry = squel.select()
        .field('estado.nome', 'name')
        .field('escola.tipo_localizacao', 'location')
        .field('COUNT(DISTINCT(escola.cod_entidade))', 'total')
        .field('escola.ano_censo', 'census_year')
        .from('escola')
        .from('estado')
        .where('escola.fk_estado_id = estado.pk_estado_id')
        .where(`escola.fk_estado_id = ${stateId}`)
        .where(`escola.ano_censo IN (${censusYearQry})`)
        .where('escola.id_tipo_turma = 0')
        .group('estado.nome')
        .group('escola.tipo_localizacao')
        .group('escola.ano_censo');

    const queries = [totalSchoolsQry, schoolsPerLocationQry].map((qry) => {
        return sqlQuery(qry.toString());
    });

    // execute all queries concurrently
    Promise.all(queries).then((queryResults) => {
        req.result = [];
        for(let result of queryResults) {
            log.debug(result);
            for(let row of result) {
                row.location = locationIdToStr(row.location);
                req.result.push(row);
            }
        }
        next();
    }).catch((error) => {
        log.error(`[SQL query error] ${error}`);
        next(error);
    });
}, response('location'));

locationApp.get('/educational/city/:id', (req, res, next) => {
    const cityId = parseInt(req.params.id, 10);

    const censusYearQry = squel.select()
        .field('MAX(escola.ano_censo)', 'ano_censo')
        .from('escola')
        .toString();

    const totalSchoolsQry = squel.select()
        .field('municipio.nome', 'name')
        .field('0', 'location')
        .field('COUNT(DISTINCT(escola.cod_entidade))', 'total')
        .field('escola.ano_censo', 'census_year')
        .from('escola')
        .from('municipio')
        .where('escola.fk_municipio_id = municipio.pk_cod_ibge')
        .where(`escola.fk_municipio_id = ${cityId}`)
        .where(`escola.ano_censo IN (${censusYearQry})`)
        .where('escola.id_tipo_turma = 0')
        .group('municipio.nome')
        .group('escola.ano_censo');

    const schoolsPerLocationQry = squel.select()
        .field('municipio.nome', 'name')
        .field('escola.tipo_localizacao', 'location')
        .field('COUNT(DISTINCT(escola.cod_entidade))', 'total')
        .field('escola.ano_censo', 'census_year')
        .from('escola')
        .from('municipio')
        .where('escola.fk_municipio_id = municipio.pk_cod_ibge')
        .where(`escola.fk_municipio_id = ${cityId}`)
        .where(`escola.ano_censo IN (${censusYearQry})`)
        .where('escola.id_tipo_turma = 0')
        .group('municipio.nome')
        .group('escola.tipo_localizacao')
        .group('escola.ano_censo');

    const queries = [totalSchoolsQry, schoolsPerLocationQry].map((qry) => {
        return sqlQuery(qry.toString());
    });

    // execute all queries concurrently
    Promise.all(queries).then((queryResults) => {
        req.result = [];
        for(let result of queryResults) {
            log.debug(result);
            for(let row of result) {
                row.location = locationIdToStr(row.location);
                req.result.push(row);
            }
        }
        next();
    }).catch((error) => {
        log.error(`[SQL query error] ${error}`);
        next(error);
    });
}, response('location'));

module.exports = locationApp;
